# -*- encoding: utf-8 -*-
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# @brief Code generated by Mobius. Do not edit
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
EXTENSION_ID = 'datasource-dossier'
EXTENSION_NAME = 'Datasource Dossier'
EXTENSION_AUTHOR = 'Vladimir Santos'
EXTENSION_VERSION = '0.1.0'
EXTENSION_DESCRIPTION = 'Logicube Dossier datasource support'

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# Mobius Forensic Toolkit
# Copyright (C) 2008,2009,2010,2011 Eduardo Aguiar
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
import re
import os.path
import time
import urllib

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# @brief constants
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
EXP_OPERATING_MODE = re.compile (r'Operating Mode: (.*?):')
EXP_DRIVE_MODEL = re.compile (r'Model : (.*?)\|')
EXP_DRIVE_SERIAL = re.compile (r'Serial: (.*?)\|')
EXP_SKIPPED_SECTORS = re.compile (r'Skipped Sectors\[1\]: ([0-9]+)')

STATUS_GOOD, STATUS_BAD, STATUS_UNKNOWN = range (3)
SECTOR_SIZE = 512
SECTORS_PER_READ = 128
BUFFER_SIZE = SECTOR_SIZE * SECTORS_PER_READ

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# @brief convert time from uint64 to string
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
def strftime (utime):
  if utime != 0:
    strtime = time.strftime ('%Y-%m-%d %T', time.localtime (utime))
  else:
    strtime = ''

  return strtime

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# @brief Dossier Reader
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
class Reader (object):

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief initialize reader
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def __init__ (self, datasource):
    uri = gdata.mediator.call ('object.get-attribute', datasource, 'uri')
    self.__prefix = uri.rsplit ('.')[0]
    self.__pos = 0
    self.__stream = None
    self.__segment_size = -1
    self.__image_size = 0

    # find segment_size and image_size
    i = 1
    path = self.__get_segment_file (i)

    while os.path.exists (path):
      size = os.path.getsize (path)
      self.__image_size += size

      if i == 1:
        self.__segment_size = size

      i += 1
      path = self.__get_segment_file (i)

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief del reader
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def __del__ (self):
    self.close ()

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @deprecated walk through all segment files
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def __iter__ (self):
    data = self.read (BUFFER_SIZE)
    while data:
      yield STATUS_UNKNOWN, data
      data = self.read (BUFFER_SIZE)

    self.close ()

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief close reader
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def close (self):
    if self.__stream:
      self.__stream.close ()
      self.__segment_file = -1
      self.__image_size = 0

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief set reading position
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def seek (self, pos, whence):
    if whence == 0:	# from beginning of file
      abs_pos = pos

    elif whence == 1:	# from current position
      abs_pos = self.__pos + pos

    elif whence == 2:	# from end of file
      abs_pos = self.__image_size - 1 + pos

    if abs_pos >= 0 and abs_pos < self.__image_size:
      self.__pos = abs_pos
      self.__stream = None

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief get the current reading position
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def tell (self):
    return self.__pos

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief read bytes
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def read (self, siz):
    if not self.__stream:
      self.__set_stream ()

    data = self.__stream.read (siz)
    self.__pos += len (data)
    siz = siz - len (data)

    while siz > 0 and self.__pos < self.__image_size:
      self.__set_stream ()

      tmp = self.__stream.read (siz)
      self.__pos += len (tmp)
      siz = siz - len (tmp) 
      data += tmp

    return data

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief return segment file from idx
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def __get_segment_file (self, idx):
    return '%s.%03d' % (self.__prefix[7:], idx)

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief set stream to read current position
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def __set_stream (self):
    idx = (self.__pos // self.__segment_size) + 1
    pos = self.__pos % self.__segment_size

    path = self.__get_segment_file (idx)
    self.__stream = open (path)
    self.__stream.seek (pos, 0)

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief return stream size
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def __get_size (self):
    return self.__image_size

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # properties
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  size = property (__get_size, None)

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# @brief Dossier datasource support
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
class DossierHandler (object):

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief initialize object
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def __init__ (self):
    self.type = 'dossier'
    self.description = 'Logicube Dossier image file'
    self.extensions = ['log']
    self.Reader = Reader
    self.default_item_category = 'harddisk'
    self.is_exportable = False

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief verify if it is a dossier .log file
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def signature_found (self, uri):
    path = urllib.unquote (uri)[7:]

    fp = open (path)
    data = fp.read (128)
    fp.close ()

    return 'Forensic Dossier' in data

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief retrieve metadata from uri
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def retrieve_metadata (self, uri):
    path = urllib.unquote (uri)[7:]

    metadata = {}
    metadata['type'] = 'dossier'
    metadata['uri'] = uri

    # stat log file
    statinfo = os.stat (path)
    metadata['acquisition_time'] = strftime (statinfo.st_mtime)

    # read log file
    fp = open (path)
    data = fp.read ()
    fp.close ()

    # parse log file
    skipped_sectors = set ()
    state = 0

    for line in data.replace ('\r', '').split ('\n'):

      # initial state
      if state == 0:
        match = EXP_OPERATING_MODE.search (line)
        if match:
          operating_mode = match.group (1).lower ()

        elif 'SOURCE DRIVE' in line:
          state = 1

        elif 'Skipped Sector Addresses[1]' in line:
          state = 3

        elif 'Forensic Dossier' in line:
          metadata['dossier_serial'] = line[37:43].strip ()
          metadata['dossier_version'] = line[56:65].strip ()

      # "SOURCE DRIVE" state
      elif state == 1:
        match = EXP_DRIVE_MODEL.search (line)
        if match:
          metadata['drive_model'] = match.group (1).strip ()

        match = EXP_DRIVE_SERIAL.search (line)
        if match:
          metadata['drive_serial'] = match.group (1).strip ()

        if 'Total Sectors' in line:
          state = 2

      # "Total Sectors" state
      elif state == 2:
        total_sectors = int (line[2:16].strip ())
        metadata['size'] = total_sectors * SECTOR_SIZE
        metadata['total_sectors'] = total_sectors
        state = 0

      # "Skipped Sector Addresses" state
      elif state == 3:
        if 'Skipped Sectors[1]:' in line:
          match = EXP_SKIPPED_SECTORS.search (line)
          if match:
            metadata['bad_sectors'] = int (match.group (1))
          state = 0
        else:
          data = set (line.strip ()[1:-1].strip ().split ())
          skipped_sectors.update (data)

    # set other metadata
    metadata['name'] = '%s:%s' % (metadata.get ('drive_model'), metadata.get ('drive_serial'))
    metadata['acquisition_tool'] = 'Logicube Dossier. Serial: %s. Firmware: %s'   \
         % (metadata.get ('dossier_serial'), metadata.get ('dossier_version'))
    metadata['bad_sector_list'] = skipped_sectors

    return metadata

  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  # @brief set item attributes
  # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  def set_item_attributes (self, item, datasource):
    category = gdata.mediator.call ('item.get-attribute', item, 'category')
    if category != 'harddisk':
      return

    drive_model = gdata.mediator.call ('object.get-attribute', datasource, 'drive_model')
    drive_serial = gdata.mediator.call ('object.get-attribute', datasource, 'drive_serial')
    size = gdata.mediator.call ('object.get-attribute', datasource, 'size')
    total_sectors = gdata.mediator.call ('object.get-attribute', datasource, 'total_sectors')
    part_id = drive_model.split (' ')[-1].split ('-')[0]

    gdata.mediator.call ('item.set-attribute', item, 'part_id', part_id)
    gdata.mediator.call ('item.set-attribute', item, 'serial', drive_serial)
    gdata.mediator.call ('item.set-attribute', item, 'lba', total_sectors)
    gdata.mediator.call ('item.set-attribute', item, 'real_capacity', size)
    gdata.mediator.call ('item.expand-value-masks', item)

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# @brief start function
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
def pvt_start ():
  gdata.mediator.call ('datasource.add-handler', DossierHandler ())

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# @brief stop function
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
def pvt_stop ():
  gdata.mediator.call ('datasource.remove-handler', 'dossier')

# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# API initialization
# =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
def pvt_start_api ():
  pass
